==> sqlitedbms-0.2.2.tar.gz: sqlitedbms-0.2.2/src/backend/main.c <==
#include "include/common.h"

struct config_t		config;

/* http */
regex_t			reg_protocol;
regex_t			reg_method;
regex_t			reg_request;

/* extended sql */
regex_t			reg_admin_dump;
regex_t			reg_admin_show;
regex_t			reg_admin_desc;

/* extended sql for administrator */
regex_t			reg_admin_kill;
regex_t			reg_admin_createdb;
regex_t			reg_admin_dropdb;
regex_t			reg_admin_createuser;
regex_t			reg_admin_dropuser;

regex_t			reg_admin_grant;
regex_t			reg_admin_revoke;
regex_t			reg_admin_grant_create;
regex_t			reg_admin_revoke_create;

struct stat_t		thread_stat;

const char		*libversion;

int main( int argc, char *argv[] )
{
	sock_t		server_addr;
	int		server_fd;
	struct pollfd	server_poll;
	int		server_poll_res;

	sock_t		client_addr;
	int		client_addr_size;
	int		client_fd;
	int		client_fd_opt;

	pthread_t	client_thread;

	pthread_attr_t	thread_attr;

	struct passwd	*pwd;
	uid_t		uid;
	gid_t		gid;
	
	struct rlimit	limit_r;

	char		admin_id[64];
	char		admin_pass[64];
	char		admin_confirm[8];
	char		*admin_tmp;
	short		admin_process;


	init_config();
	if ( parse_config( argc, argv ) == -1 )
	{
		//-h
		return 0;
	}

	chdir( __PREFIX );

	limit_r.rlim_cur = 1024*16;
	limit_r.rlim_max = 1024*16;
	setrlimit( RLIMIT_NOFILE, &limit_r );
	setrlimit( RLIMIT_NPROC, &limit_r );

	libversion = db_version();

	if ( check_catalog_exists() != TRUE )
	{
		admin_process = FALSE;
		admin_confirm[0] = '_';

		while ( admin_process == FALSE )
		{
			memset( admin_id, 0, 64 );
			memset( admin_pass, 0, 64 );

			printf( "** Catalog files not found. Insert admin id and password for initialize catalog.\n   Administrator ID: " );
			fflush( stdout );
			fgets( admin_id, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_id, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_id, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			if ( strlen( admin_id ) <= 0 )
			{
				printf( "** Insert administrator ID.\n" );
				continue;
			}

			printf( "   Administrator password: " );
			fflush( stdout );
			fgets( admin_pass, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_pass, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_pass, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';

			if ( strlen( admin_pass ) <= 0 )
			{
				printf( "** Insert administrator password.\n" );
				continue;
			}

			while ( admin_confirm[0] != 'y' || admin_confirm[0] != 'Y' || admin_confirm[0] != 'n' || admin_confirm[0] != 'N' )
			{
				printf( "   Administrator's ID: %s, password: %s\n   Right? (y/n): ", admin_id, admin_pass );
				fflush( stdout );
				fgets( admin_confirm, 8, stdin );
				fflush( stdin );
				if ( admin_confirm[0] == 'y' || admin_confirm[0] == 'Y' )
				{
					admin_process = TRUE;
					break;
				}
				else if ( admin_confirm[0] == 'n' || admin_confirm[0] == 'N' )
				{
					printf( "\n" );
					break;
				}
			}
		}
	}else{
		memset( admin_id, 0, 64 );
		memset( admin_pass, 0, 64 );
	}

	printf("** Startup SQLiteDBMS.\n");

#ifndef __DEBUG
	pwd = getpwnam( config.server_user );
	if ( pwd == NULL )
	{
		perror( "Unknown user" );
		exit( 1 );
	}
	uid = pwd->pw_uid;
	gid = pwd->pw_gid;

	switch ( fork() )
	{
		case 0: break;
		case -1:
			perror( "fork() failed" );
		default:
			exit(1);
	}

	(void) setsid();

	if ( getuid() == 0 )
	{
		if ( setgroups( 0, NULL ) < 0 )
		{
			perror( "Set groups error" );
			exit( 1 );
		}

		if ( setgid( gid ) < 0 )
		{
			perror( "Set gid error" );
			exit( 1 );
		}

		if ( initgroups( config.server_user, gid ) < 0 )
			perror( "Initgroups error" );

		if ( setuid( uid ) < 0 )
		{
			perror( "Set uid error" );
			exit( 1 );
		}
	}
#endif

	(void) signal( SIGPIPE, SIG_IGN );
	(void) signal( SIGINT, safe_exit );

	if ( auth_init( admin_id, admin_pass ) != SQLITE_OK )
		return -1;

	pthread_mutex_init( &thread_stat.lock, NULL );
	thread_stat.first = NULL;
	thread_stat.count = 0;

	server_fd = server_sock_init( &server_addr, config.server_port, 1000 );
	if (server_fd <= 0)
	{
		perror( "Cannot open server socket" );
		return -1;
	}

	server_poll.fd = server_fd;
	server_poll.events = POLLIN;

	client_fd_opt = 1;

	printf("** Preparing HTTP.\n");

	regcomp( &reg_protocol,	"(\\\n\\\n)|(\\\r\\\n\\\r\\\n)|(\\\r\\\r)", REG_EXTENDED );
	regcomp( &reg_method,	"^([A-Z]+) \\/([^ ]*) HTTP\\/1\\.[0-1]", REG_EXTENDED );
	regcomp( &reg_request,	"([a-zA-z-]+)\\s?:\\s?([^\\\n\\\r]+)", REG_EXTENDED );

	printf("** Preparing extended SQL.\n");

	regcomp( &reg_admin_dump,	"DUMP\\s+TO\\s+((MIRROR )(FILE) '([^']*)'|(CLIENT))", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_show,	"SHOW\\s+([a-zA-Z0-9_]+)[ ]?([a-zA-Z0-9_\"]*)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_desc,	"DESC\\s+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );

	regcomp( &reg_admin_kill,	"KILL\\s+([0-9]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_createdb,	"CREATE\\s+DATABASE ([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_dropdb,	"DROP\\s+DATABASE\\s+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_createuser,	"CREATE\\s+USER\\s+([a-zA-Z0-9_\"]+)\\s+([a-zA-Z0-9_\"]+)\\s*(SUPERUSER)?", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_dropuser,	"DROP\\s+USER\\s+([a-zA-Z0-9_\"]+)\\s*([a-zA-Z0-9_\"]*)", REG_EXTENDED | REG_ICASE );

	regcomp( &reg_admin_grant,
		"GRANT\\s+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)\\s+ON\\s+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_revoke,
		"REVOKE\\s+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)\\s+ON\\s+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_grant_create,
		"GRANT\\s+CREATE\\s+(ALL|TABLE|VIEW|INDEX|TRIGGER)\\s+ON\\s+([a-zA-Z0-9_\"*]+)\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_revoke_create,
		"REVOKE\\s+CREATE\\s+(ALL|TABLE|VIEW|INDEX|TRIGGER)\\s+ON\\s+([a-zA-Z0-9_\"*]+)\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );

	pthread_attr_init( &thread_attr );
	pthread_attr_setdetachstate( &thread_attr, PTHREAD_CREATE_DETACHED );

	client_addr_size = sizeof( client_addr.sa );

	printf("** SQLiteDBMS is ready.\n");

	while ( 1 )
	{
		server_poll_res = poll( &server_poll, 1, config.poll_timer );
		if ( server_poll_res > 0 )
		{
			client_fd = accept( server_fd, &client_addr.sa, &client_addr_size );

			if ( client_fd < 0 )
				continue;

			if ( fcntl( client_fd, F_SETFL, O_RDWR|O_NONBLOCK|O_ASYNC|O_NDELAY ) < 0 )
			{
				close( client_fd );
				continue;
			}

#ifdef __DEBUG
			printf("***** New client.\n");
#endif

			setsockopt( client_fd, IPPROTO_TCP, TCP_NODELAY, &client_fd_opt, sizeof(client_fd_opt) );

			if ( pthread_create( &client_thread, &thread_attr, client, &client_fd ) != 0 )
				close( client_fd );
		}
	}

	return 0;
}

short check_catalog_exists()
{
	extern struct config_t	config;
	char			filepath[1024];
	int			fd;

	snprintf( filepath, 1024, "%s/catalog.user.superdb", config.db_path );
	fd = open( filepath, O_RDONLY );
	if ( fd <= 0 )
		return FALSE;
	close( fd );
	return TRUE;
}

void safe_exit( int sig )
{
	extern struct db_t	db_auth;
	extern struct auth_t	auth;

	if ( auth.user.table != NULL )
		_db_free_table( auth.user.table );
	if ( auth.db.table != NULL )
		_db_free_table( auth.db.table );
	if ( auth.object.table != NULL )
		_db_free_table( auth.object.table );
	db_close( &db_auth );

	regfree( &reg_protocol );
	regfree( &reg_method );
	regfree( &reg_request );
	regfree( &reg_admin_dump );
	regfree( &reg_admin_show );
	regfree( &reg_admin_desc );
	regfree( &reg_admin_kill );
	regfree( &reg_admin_createdb );
	regfree( &reg_admin_dropdb );
	regfree( &reg_admin_createuser );
	regfree( &reg_admin_dropuser );
	regfree( &reg_admin_grant );
	regfree( &reg_admin_revoke );
	regfree( &reg_admin_grant_create );
	regfree( &reg_admin_revoke_create );

	printf("** SQLiteDBMS is shutdown.\n");
	exit( 0 );
}

==> sqlitedbms-0.3.0.tar.gz: sqlitedbms-0.3.0/src/backend/main.c <==
#include "include/common.h"

struct config_t		config;

/* extended sql */
regex_t			reg_admin_dump;
regex_t			reg_admin_show;
regex_t			reg_admin_desc;

/* extended sql for administrator */
regex_t			reg_admin_kill;
regex_t			reg_admin_createdb;
regex_t			reg_admin_dropdb;
regex_t			reg_admin_createuser;
regex_t			reg_admin_dropuser;
regex_t			reg_admin_alteruser;

regex_t			reg_admin_grant;
regex_t			reg_admin_revoke;
regex_t			reg_admin_grant_create;
regex_t			reg_admin_revoke_create;

struct stat_t		thread_stat;

const char		*libversion;

int main( int argc, char *argv[] )
{
	sock_t		server_addr;
	int		server_fd;
	struct pollfd	server_poll;
	int		server_poll_res;

	sock_t		client_addr;
	int		client_addr_size;
	int		client_fd;
	int		client_fd_opt;

	pthread_t	client_thread;

	pthread_attr_t	thread_attr;

	struct passwd	*pwd;
	uid_t		uid;
	gid_t		gid;
	
	struct rlimit	limit_r;

	char		admin_id[64];
	char		admin_pass[64];
	char		admin_confirm[8];
	char		*admin_tmp;
	short		admin_process;


	init_config();
	if ( parse_config( argc, argv ) == -1 )
	{
		//-h
		return 0;
	}

	chdir( __PREFIX );

	limit_r.rlim_cur = 1024*16;
	limit_r.rlim_max = 1024*16;
	setrlimit( RLIMIT_NOFILE, &limit_r );
	setrlimit( RLIMIT_NPROC, &limit_r );

	libversion = db_version();

	if ( check_catalog_exists() != TRUE )
	{
		admin_process = FALSE;
		admin_confirm[0] = '_';

		while ( admin_process == FALSE )
		{
			memset( admin_id, 0, 64 );
			memset( admin_pass, 0, 64 );

			printf( "** Catalog files not found. Insert admin id and password for initialize catalog.\n   Administrator ID: " );
			fflush( stdout );
			fgets( admin_id, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_id, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_id, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			if ( strlen( admin_id ) <= 0 )
			{
				printf( "** Insert administrator ID.\n" );
				continue;
			}

			printf( "   Administrator password: " );
			fflush( stdout );
			fgets( admin_pass, 64, stdin );
			fflush( stdin );

			admin_tmp = strchr( admin_pass, '\n' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';
			admin_tmp = strchr( admin_pass, '\r' );
			if ( admin_tmp != NULL )
				*admin_tmp = '\0';

			if ( strlen( admin_pass ) <= 0 )
			{
				printf( "** Insert administrator password.\n" );
				continue;
			}

			while ( admin_confirm[0] != 'y' || admin_confirm[0] != 'Y' || admin_confirm[0] != 'n' || admin_confirm[0] != 'N' )
			{
				printf( "   Administrator's ID: %s, password: %s\n   Right? (y/n): ", admin_id, admin_pass );
				fflush( stdout );
				fgets( admin_confirm, 8, stdin );
				fflush( stdin );
				if ( admin_confirm[0] == 'y' || admin_confirm[0] == 'Y' )
				{
					admin_process = TRUE;
					break;
				}
				else if ( admin_confirm[0] == 'n' || admin_confirm[0] == 'N' )
				{
					printf( "\n" );
					break;
				}
			}
		}
	}else{
		memset( admin_id, 0, 64 );
		memset( admin_pass, 0, 64 );
	}

	printf("** Startup SQLiteDBMS.\n");

#ifndef __DEBUG
	pwd = getpwnam( config.server_user );
	if ( pwd == NULL )
	{
		perror( "Unknown user" );
		exit( 1 );
	}
	uid = pwd->pw_uid;
	gid = pwd->pw_gid;

	switch ( fork() )
	{
		case 0: break;
		case -1:
			perror( "fork() failed" );
		default:
			exit(1);
	}

	(void) setsid();

	if ( getuid() == 0 )
	{
		if ( setgroups( 0, NULL ) < 0 )
		{
			perror( "Set groups error" );
			exit( 1 );
		}

		if ( setgid( gid ) < 0 )
		{
			perror( "Set gid error" );
			exit( 1 );
		}

		if ( initgroups( config.server_user, gid ) < 0 )
			perror( "Initgroups error" );

		if ( setuid( uid ) < 0 )
		{
			perror( "Set uid error" );
			exit( 1 );
		}
	}
#endif

	(void) signal( SIGPIPE, SIG_IGN );
	(void) signal( SIGINT, safe_exit );

	if ( auth_init( admin_id, admin_pass ) != SQLITE_OK )
		return -1;

	pthread_mutex_init( &thread_stat.lock, NULL );
	thread_stat.first = NULL;
	thread_stat.count = 0;

	server_fd = server_sock_init( &server_addr, config.server_port, 1000 );
	if (server_fd <= 0)
	{
		perror( "Cannot open server socket" );
		return -1;
	}

	server_poll.fd = server_fd;
	server_poll.events = POLLIN;

	client_fd_opt = 1;

	printf("** Preparing extended SQL.\n");

	regcomp( &reg_admin_dump,	"DUMP\\s+TO\\s+((MIRROR )(FILE) '([^']*)'|(CLIENT))", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_show,	"SHOW\\s+([a-zA-Z0-9_]+)[ ]?([a-zA-Z0-9_\"]*)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_desc,	"DESC\\s+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );

	regcomp( &reg_admin_kill,	"KILL\\s+([0-9]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_createdb,	"CREATE\\s+DATABASE ([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_dropdb,	"DROP\\s+DATABASE\\s+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_createuser,	"CREATE\\s+USER\\s+([a-zA-Z0-9_\"]+)\\s+([a-zA-Z0-9_\"]+|'[^']+')\\s*(SUPERUSER)?", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_dropuser,	"DROP\\s+USER\\s+([a-zA-Z0-9_\"]+)", REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_alteruser,	"ALTER\\s+USER\\s+([a-zA-Z0-9_\"]+)\\s+SET\\s+(((PASSWORD)\\s+([a-zA-Z0-9_\"]+|'[^']+'))|((NO)?(SUPERUSER)))", REG_EXTENDED | REG_ICASE );

	regcomp( &reg_admin_grant,
		"GRANT\\s+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)\\s+ON\\s+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_revoke,
		"REVOKE\\s+(ALL|SELECT|INSERT|UPDATE|DELETE|ALTER|TRIGGER|SHOW)\\s+ON\\s+([a-zA-Z0-9_\"*]+)[.]?([a-zA-Z0-9_\"*]+)?\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_grant_create,
		"GRANT\\s+CREATE\\s+(ALL|TABLE|VIEW|INDEX|TRIGGER)\\s+ON\\s+([a-zA-Z0-9_\"*]+)\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );
	regcomp( &reg_admin_revoke_create,
		"REVOKE\\s+CREATE\\s+(ALL|TABLE|VIEW|INDEX|TRIGGER)\\s+ON\\s+([a-zA-Z0-9_\"*]+)\\s+TO\\s+(([a-zA-Z0-9_\"]+)|PUBLIC)",
		REG_EXTENDED | REG_ICASE );

	pthread_attr_init( &thread_attr );
	pthread_attr_setdetachstate( &thread_attr, PTHREAD_CREATE_DETACHED );

	client_addr_size = sizeof( client_addr.sa );

	printf("** SQLiteDBMS is ready.\n");

	while ( 1 )
	{
		server_poll_res = poll( &server_poll, 1, config.poll_timer );
		if ( server_poll_res > 0 )
		{
			client_fd = accept( server_fd, &client_addr.sa, &client_addr_size );

			if ( client_fd < 0 )
				continue;

			if ( fcntl( client_fd, F_SETFL, O_RDWR|O_NONBLOCK|O_ASYNC|O_NDELAY ) < 0 )
			{
				close( client_fd );
				continue;
			}

#ifdef __DEBUG
			printf("***** New client.\n");
#endif

			setsockopt( client_fd, IPPROTO_TCP, TCP_NODELAY, &client_fd_opt, sizeof(client_fd_opt) );

			if ( pthread_create( &client_thread, &thread_attr, client, &client_fd ) != 0 )
				close( client_fd );
		}
	}

	return 0;
}

short check_catalog_exists()
{
	extern struct config_t	config;
	char			filepath[1024];
	int			fd;

	snprintf( filepath, 1024, "%s/catalog.user.superdb", config.db_path );
	fd = open( filepath, O_RDONLY );
	if ( fd <= 0 )
		return FALSE;
	close( fd );
	return TRUE;
}

void safe_exit( int sig )
{
	extern struct db_t	db_auth;
	extern struct auth_t	auth;

	if ( auth.user.table != NULL )
		_db_free_table( auth.user.table );
	if ( auth.db.table != NULL )
		_db_free_table( auth.db.table );
	if ( auth.object.table != NULL )
		_db_free_table( auth.object.table );
	db_close( &db_auth );

	regfree( &reg_admin_dump );
	regfree( &reg_admin_show );
	regfree( &reg_admin_desc );
	regfree( &reg_admin_kill );
	regfree( &reg_admin_createdb );
	regfree( &reg_admin_dropdb );
	regfree( &reg_admin_createuser );
	regfree( &reg_admin_dropuser );
	regfree( &reg_admin_grant );
	regfree( &reg_admin_revoke );
	regfree( &reg_admin_grant_create );
	regfree( &reg_admin_revoke_create );

	printf("** SQLiteDBMS is shutdown.\n");
	exit( 0 );
}
